import time

from threads import QueueThread

global _log
global _request

_log = None
_request = None

class StartLogError(Exception):
    def __init__(self):
        Exception.__init__(self, "The log has not been started")

class _ENTRY_TYPE:
    INFO =    0
    SUCCESS = 1
    BOLD =    2
    WARN =    3
    ERROR =   4

_PRINT_COLOR = {
    _ENTRY_TYPE.INFO:    "%s",
    _ENTRY_TYPE.SUCCESS: "\033[92m%s\033[0m", # Green
    _ENTRY_TYPE.BOLD:    "\033[94m%s\033[0m", # Blue
    _ENTRY_TYPE.WARN:    "\033[93m%s\033[0m", # Yellow
    _ENTRY_TYPE.ERROR:   "\033[91m%s\033[0m", # Red
}

_LOG_ABRV = {
    _ENTRY_TYPE.INFO:    "   ",
    _ENTRY_TYPE.SUCCESS: "+++",
    _ENTRY_TYPE.BOLD:    ">>>",
    _ENTRY_TYPE.WARN:    "WRN",
    _ENTRY_TYPE.ERROR:   "ERR",
}

class _LogEntry:
    def __init__(self, entrytype, data):
        self.timestamp = time.time()
        self.entrytype = entrytype
        self.data = data

class LogWorker(QueueThread):
    def __init__(self, filename,
                 initFunc = None,
                 cleanupFunc = None,
                 logFmtFunc = None,
                 printFmtFunc = None):

        QueueThread.__init__(self)
        self.logfile = open(filename, "w")
        self.unloggedLines = []
        self.initFunc = initFunc
        self.cleanupFunc = cleanupFunc

        if logFmtFunc:
            self.logFmtFunc = logFmtFunc

        if printFmtFunc:
            self.printFmtFunc = printFmtFunc

    def startLogFile(self, logfile):
        self.logfile = logfile
        for line in self.unloggedLines:
            self.logfile.write(line)
        self.unloggedLines = []

    def init(self):
        if self.initFunc:
            self.initFunc()

    def cleanup(self):
        if self.logfile:
            self.logfile.close()
            self.logfile = None
        if self.cleanupFunc:
            self.cleanupFunc()

    def requestHandler(self, logentry):
        print self.printFmtFunc(logentry)

        text = self.logFmtFunc(logentry)
        if self.logfile:
            self.logfile.write(text)
        else:
            self.unloggedLines.append(text)

    def request(self, data, entrytype = _ENTRY_TYPE.INFO):
        logentry = _LogEntry(entrytype, data)
        QueueThread.request(self, logentry, callback = self.requestHandler)
        
###############################################################################
# Plain Text Helpers
###############################################################################

def formatTimestamp(timestamp):
    return (time.strftime("%m/%d/%y-%H:%M:%S.",  time.localtime(timestamp)) +
            "%06d" %(1000000* (timestamp - int(timestamp))))

def plaintextLogFmtFunc(logentry):
    timestamp = formatTimestamp(logentry.timestamp)
    
    try:
        abrv = _LOG_ABRV[logentry.entrytype]
    except LookupError as e:
        abrv = _LOG_ABRV[_ENTRY_TYPE.INFO]

    result = ""
    for line in str(logentry.data).split("\n"):
        result = result + "[%s] %s %s\n" %(timestamp, abrv, line)

    return result
                              
def consolePrintFmtFunc(logentry):
    timestamp = formatTimestamp(logentry.timestamp)
    
    try:
        fmt = _PRINT_COLOR[logentry.entrytype]
    except LookupError as e:
        fmt = _PRINT_COLOR[_ENTRY_TYPE.INFO]
        
    result = ""
    prefix = ""
    for line in str(logentry.data).split("\n"):
        result = result + "%s[%s] %s" %(prefix, timestamp, fmt %(line))
        prefix = "\n"

    return result

def plaintextRequest(data, entrytype = _ENTRY_TYPE.INFO):
    global _log
    if not _log:
        raise StartLogError()
    _log.request(str(data), entrytype)

###############################################################################
# Module API
###############################################################################

def startPlaintextLog(filename):
    global _log
    global _request
    _log = LogWorker(filename,
                     logFmtFunc = plaintextLogFmtFunc,
                     printFmtFunc = consolePrintFmtFunc)
    _request = plaintextRequest
    _log.start()

def info(data):
    global _request
    if not _request:
        raise StartLogError()
    _request(data, _ENTRY_TYPE.INFO)

def success(data):
    global _request
    if not _request:
        raise StartLogError()
    _request(data, _ENTRY_TYPE.SUCCESS)

def bold(data):
    global _request
    if not _request:
        raise StartLogError()
    _request(data, _ENTRY_TYPE.BOLD)

def warn(data):
    global _request
    if not _request:
        raise StartLogError()
    _request(data, _ENTRY_TYPE.WARN)

def error(data):
    global _request
    if not _request:
        raise StartLogError()
    _request(data, _ENTRY_TYPE.ERROR)
